/*
 *	ezinstaller.c
 *	Author: Tim Niiler
 * 	Date: 3/12/04
 *	Version: 0.1.2
 *	License: GPL
 *		NOTE: Companies or individuals that either do not "believe in" the GPL
 *		or do not abide by it MAY NOT use this code for ANY purpose as they will
 *		be violating copyright law. Otherwise, so long as you agree to the GPL
 *		have at it, you may modify this code as you wish with the understanding
 *		that you keep all comments and credits intact.
 *
 *	Purpose:
 *		This code is designed to act as an easy way to install Linux packages
 *		on any distribution.  Ahem...well, at least on Vector Linux and Slackware
 *		to start with.  But I am trying to write the code in such a way as to 
 *		minimize the distro dependencies in the C portion of it.
 *	
 *		As such, the code calls several distro dependent scripts which the users
 *		may find much easier to modify to fit their distro of choice.  Again, at
 *		the time of this writing, these scripts have only been written for 
 *		Vector Linux and Slackware.
 *	
 *	Disclaimers:
 *		As it has been years since I used C (and I never really liked it), at this 
 *		point I can't guarantee the stability of the code.  But darn it, I'm trying
 *		to get it to the point where it's robust.  As such, any help fellow developers
 * 		can give would be most welcome, especially concerning PROPER and CONVENTIONAL
 *		use of pointers and the like.
 *
 *		As per usual, you use this code at your own risk.
 */
#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <string.h>

gchar *array[500];
gchar *array1[500];
gchar array2[500]=" ";
gchar array3[500]=" ";
gchar ftpsite[500]="ftp://ftp.ibiblio.org/pub/linux/distributions/vectorlinux/veclinux-4.0/packages/";
gchar manifestsite[500]="ftp://ftp.ibiblio.org/pub/linux/distributions/vectorlinux/veclinux-soho-4.0/stable/niiler/installer/";

gchar setDebug[6] = "TRUE";
gchar cleanUp[6] = "TRUE";
gchar internetInstall[6] = "TRUE";
gchar isDialog1OK[5] = "YES";
gchar isDialog2OK[5] = "YES";
gchar isDialog3OK[5] = "YES";
gchar isDialog4OK[5] = "YES";
gchar isDialogOpen[6] = "TRUE";
gchar iconInstall[6] = "TRUE";
gchar packageName[50];
gchar packageLoc[50];
gchar packageIcon[50];
int newPackage = 0;

int arraySize;
int ftpsize;
int nPackages;
int allPackages;
int okInstall = 1;

/* Add memory here */
struct menuConts
{
        gchar cat[50];
	gchar md5[500];
        gchar xpix[50];
	gchar loc[50];
        gchar meth[50];
        gchar text[500];
        gchar where[3500];
} programs[100]; 

struct ftpStuff
{
	gchar where[1000];
} ftparray[100]; 

struct ftpStuff ftpout[100];
struct ftpStuff currentPackages[500];
struct ftpStuff ftpPackages[500];

union widgets {
        GtkWidget *widget;
        gpointer data;
        gchar *text;
} widgetArray[5];


struct packageThere
{
	int response;
	int which;
} packageOK; 


void initArr();
static int showError( gchar * text);
void makeKDEIcon(gchar * progLoc, gchar * iconLoc, gchar * progName);
void makeIceWMIcon(gchar * progLoc, gchar * progName);
void setDialogRules(GtkWidget * widget, gpointer data);
void setDialogIsOpen (GtkWidget * widget, gpointer data);
void substr( char sIn[], char sOut[], int start);
void deblank( char * l );
void writeFile( char inputFile[], char text[]);
static int doSplit(gchar *searchString, gchar *token);
static int doSplitLine(gchar *searchString, gchar *token);
static GtkWidget *xpm_label_box (gchar *xpm_filename, gchar *label_text);
int whichPackage (char * newpackage, char * oldpackage);
void isPackageInstalled(gchar *newpackage, gchar *userInput);
void getManifest();
static void installFTPpackages_4 (GtkWidget *widget, gpointer data);
static void listFTPpackages_3 (GtkWidget *widget, gpointer data);
static void listPackageTitles_2 (GtkWidget *widget, gpointer data);
static void doUninstall (GtkWidget *widget, gpointer data);
static void uninstall_1 (GtkWidget *widget, gpointer data);
static void installSingleFTPpackage_3 (GtkWidget *widget, gpointer data);
static void listPackageByName_2 (GtkWidget *widget, gpointer data);
static void browseselect ( GtkWidget *widget, GtkFileSelection *fs);
static void browseinstall( GtkWidget *widget, gpointer data);
static void destroy( GtkWidget *widget, gpointer data);
static void doInstall_1 (GtkWidget *widget, gpointer data);
static void saveprefs();
static void updateManifest_3 (GtkWidget *widget, GtkWidget *data);
static void updatePkgList (GtkWidget *widget, GtkWidget *data);
static void setmanifest_2 (GtkWidget *widget, gpointer data);
static void getmanifest_2 (GtkWidget *widget, gpointer data);
static void localManifestSelect_3 (GtkWidget *widget, GtkFileSelection *fs);
static void chooseLocalManifest_2 (GtkWidget *widget, gpointer data);
static void setOtherPreferences_2 (GtkWidget *widget, gpointer data);
static void localPkglistSelect_3 (GtkWidget *widget, GtkFileSelection *fs);
static void chooseLocalPkglist_2 (GtkWidget *widget, gpointer data);
static void setup_1 (GtkWidget *widget, gpointer data);



/* Now start with subroutines */

void initArr() 
{
	/* Initialize two global pointer arrays */
	*array = array2;
	*array1 = array3;
}

static int showError(gchar * text) 
{
	GtkWidget * window3;
	GtkWidget * label;

	window3= gtk_dialog_new ();
	gtk_window_set_modal(GTK_WINDOW(window3), TRUE);
	gtk_window_set_title (GTK_WINDOW (window3), "Error!");
	gtk_container_set_border_width (GTK_CONTAINER (window3), 0);
	gtk_widget_set_size_request (window3, 300, 150);

	label = gtk_label_new (text);
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window3)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_dialog_add_button (GTK_DIALOG (window3), "OK", 1);

	gint result = gtk_dialog_run (GTK_DIALOG (window3) );
	if (strcmp(setDebug,"TRUE")==0) {
		printf("Result = %i\n",result);
	}

	gtk_widget_destroy (window3);

	return 0;
}

void makeKDEIcon(gchar * progLoc, gchar * iconLoc, gchar * progName) 
{
	char outString[1000] = "[Desktop Entry]\nComment=\nComment[en_US]=\nEncoding=UTF-8\nExec=";
	char midString[500] = "\nGenericName=\nGenericName[en_US]=\nIcon=/opt/ezinstaller-0.1/icons/";
	char name[200] = "\nMimeType=\nName=";
	char name2[200] = "\nName[en_US]=";
	char endString[250] = "\nPath=\nServiceTypes=\nSwallowExec=\nSwallowTitle=\nTerminal=false\nTerminalOptions=\nType=Application\nX-KDE-SubstituteUID=false\nX-KDE-Username=";
	char commandOut[50] = "./installKDEicon.sh ";

	strcat(name,progName);
	strcat(name2,progName);
	strcat(name,name2);
	strcat(name,endString);
	strcat(midString,iconLoc);
	strcat(midString,name);
	strcat(outString,progLoc);
	strcat(outString,midString);
	writeFile("KDEIcon",outString);

	strcat(commandOut,progName);
	
	system(commandOut);

}

/* In this case create a shell script to execute program...I don't know a way to attach a specific xpm to a file in 
 * the directory .dfmdesk
 */
void makeIceWMIcon(gchar * progLoc, gchar * progName)
{
	char outString[100] = "#!/bin/sh\nexec ";
	char commandOut[50] = "./installIceWMicon.sh ";
	
	strcat(outString,progLoc);
	writeFile("IceWMIcon",outString);

	strcat(commandOut,progName);

	system(commandOut);
}

/* This subroutine is called when the level 1 dialogs are closed
 * so that only one level 1 dialog may be open at a time.
 */
void setDialogRules(GtkWidget * widget, gpointer data)
{
	if (strcmp(isDialogOpen,"FALSE")==0) {
		strcpy(isDialog1OK,"YES");
		strcpy(isDialog2OK,"YES");
		strcpy(isDialog3OK,"YES");
		strcpy(isDialog4OK,"YES");
	}
}

void setDialogIsOpen(GtkWidget * widget, gpointer data)
{
	strcpy(isDialogOpen, (char *) data);
}

/* Get substring starting at start and going to the end of the string */
void substr( char sIn[], char sOut[], int start) {
        int i;
        int j=0;
        char C;
        for (i=start;i< strlen(sIn); i++) {
                C=sIn[i];
                sOut[j]=C;
                j++;
        }
        sOut[j]='\0';
}


/* Remove blanks and initial tabs from a line */
void deblank( char * l )
{
	int i;
	char *p;
   
	i = strspn( l, "\t" ); /* find first  nonblank after tab*/
	if ( i > 0 )
	{
		for( p = l; ; p++ )
		{
			*p = *(p+i);
			if (*p == '\0') break;
		}
	}
	i = strspn( l, " " ); /* find first  nonblank after space*/
	if ( i > 0 )
	{
		for( p = l; ; p++ )
		{
			*p = *(p+i);
        		 if (*p == '\0') break;
      		}
   	}
	if ( ( ( i = strlen( l ) ) - 1 ) >= 0 )
	{
		l[i] = ( l[i] == ' ' ) ? '\0' : l[i];
	}
}

/* Write Text to a File */
void writeFile( char inputFile[], char text[])
{
	/* Write text to a file */
        FILE *inputFilePointer;
        inputFilePointer = fopen(inputFile,"w");
        fprintf(inputFilePointer,"%s",text);
	fclose(inputFilePointer);
}


/* Note: I would really like to be able to condense the next two subroutines into a single
	subroutine, but I can't figure out how to get C to pass an array back from a 
	subroutine.  When I try using pointers, unless I use the results instantly, 
	memory locations seem to be overwritten.  So much for portable code....So anyhow
	any help on this would be MUCH appreciated.
*/

static int doSplit(gchar *searchString, gchar *token)
{
	/* Split a file...in this program, this function is called for separating lines */
  int loop;

  array[0]=strtok(searchString,token);
  if(array[0]==NULL) {
	if (strcmp(setDebug,"TRUE")==0) {
	    printf("No test to search.\n");
	}
	return 0;
  } else {
	  for(loop=1;loop<500;loop++)
	   {
	    array[loop]=strtok(NULL,token);
	    if(array[loop]==NULL)
	      break;
	   }
	return loop;
  }

}

static int doSplitLine(gchar *searchString, gchar *token)
{
	/* Split a line */
  int loop;

  array1[0]=strtok(searchString,token);
  if(array1[0]==NULL) {
	if (strcmp(setDebug,"TRUE")==0) {
	    printf("No test to search.\n");
	}
	return 0;
  } else {
	for(loop=1;loop<500;loop++)
   	{
   	 array1[loop]=strtok(NULL,token);
   	 if(array1[loop]==NULL)
   	   break;
   	}
	return loop;
  }

}


/* Decide what gets included for download based on which button is pressed */
void toggle_button_callback_4 (GtkWidget *widget, int data)
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
    {
        /* If control reaches here, the toggle button is down */
	strcpy(ftpout[data].where,ftparray[data].where);

    } else {
	strcpy(ftpout[data].where,"null");
        /* If control reaches here, the toggle button is up */
    }
}



/* Create image for placement in pixmap buttons */
static GtkWidget *xpm_label_box (gchar *xpm_filename,
                                gchar *label_text)
{
	/* Copied from the GTK+ tutorial for making pixmap buttons */

        GtkWidget *box;
        GtkWidget *label;
        GtkWidget *image;

        /* Create box for image and label */
        box = gtk_hbox_new (FALSE, 0);
        gtk_container_set_border_width (GTK_CONTAINER (box), 2);

        /* Now on to the image stuff */
        image = gtk_image_new_from_file (xpm_filename);

        /* Create a label for the icon */
        label = gtk_label_new (label_text);

        /* Pack the image and label into the box */
        gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 3);
        gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 3);

        gtk_widget_show (image);
        gtk_widget_show (label);

        return box;
}


/* If a package is already on the system prompt the user to Replace, Install alongside,
 * or Cancel installation.
 */
int whichPackage (char * newpackage,
			char * oldpackage)
{
	
	static GtkWidget *window3;
	GtkWidget *label;
	char labelTxt1[1000] = "The package you have selected:\n";
	char labelTxt2[500] = "\nis similar to the package currently installed:\n";
	char labelTxt3[200] = "\nDo you want to replace the currently installed package\ninstall this package alongside the other,\nor cancel the installation of this package?";

	strcat(labelTxt2,oldpackage);
	strcat(labelTxt2,labelTxt3);
	strcat(labelTxt1,newpackage);
	strcat(labelTxt1,labelTxt2);

	window3= gtk_dialog_new ();
	gtk_window_set_modal(GTK_WINDOW(window3), TRUE);
	gtk_window_set_title (GTK_WINDOW (window3), "You must choose, Grasshopper");
	gtk_container_set_border_width (GTK_CONTAINER (window3), 0);
	gtk_widget_set_size_request (window3, 400, 150);

	label = gtk_label_new (labelTxt1);
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window3)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_dialog_add_button (GTK_DIALOG (window3), "Replace", 1);
	gtk_dialog_add_button (GTK_DIALOG (window3), "Install", 2);
	gtk_dialog_add_button (GTK_DIALOG (window3), "Cancel", 3);

	gint result = gtk_dialog_run (GTK_DIALOG (window3) );
	if (strcmp(setDebug,"TRUE")==0) {
		printf("Result = %i\n",result);
	}

	gtk_widget_destroy (window3);
	return result;

}


/* Check to see if package is already on system.  This only checks the 
 * first character string up to the first "-" character.  This has some
 * limitations and will occasionally get a false positive.  Improvements
 * are therefore needed to avoid problems like identifying gtk-engines
 * with gtk
 */
void isPackageInstalled(gchar *newpackage, gchar *userInput)
{
	gchar *result;
	int okInstall = 2;
	int j;
/* Add memory here */
/* Currently limited to 100 packages */
	gchar packname[100];
	gchar *newprefix;
	gchar *oldpackage;

	strcpy(packname,newpackage);

	if (strcmp(internetInstall,"FALSE")==0) {
		deblank(packname);
	}
	newprefix =strtok(packname,"-"); 
	if (strcmp(setDebug,"TRUE")==0) {
		printf("newpackage = %s\n",newpackage);
		printf("newprefix = %s\n",newprefix);
	}
	packageOK.which = -1;
	for (j=0; j< nPackages; j++) {
		oldpackage = currentPackages[j].where;
		result=strstr(oldpackage,newprefix);
	/*	printf("Old package = %s, result= %s\n",oldpackage,result); */ 
		if (result == NULL) {
			/* Do Nothing */
		} else {
			if (strcmp(userInput,"TRUE")==0) {
				okInstall = whichPackage(newpackage,oldpackage);
				packageOK.which = j;
			} else {
				okInstall = 3;	/* No prompt for user input */
				packageOK.which = j;
			}
			break;
		}
	}
	if (strcmp(setDebug,"TRUE")==0) {
		if (okInstall == 3) {
			printf("Package %s found!\n",currentPackages[packageOK.which].where);
		} else {
			printf("No package matched\n");
		}
	}
	packageOK.response = okInstall;
}

/* Get all the information on which packages can be installed,
 * which packages are available, and which packages are already
 * installed.
 */
void getManifest() {

	gchar manifest_version[10];
        gchar inputFileName[50]="package_manifest.txt";
	gchar pkgList[50]="pkglist.txt";
	gchar pkgManifest[50]="package_manifest.txt";
	gchar localPkgList[50]="/mnt/cdrom/pkglist.txt";
	gchar localManifest[50]="/mnt/cdrom/package_manifest.txt";
        gchar C;
/* If program segfaults, increase size of text */
/* Add memory here */
        gchar text[40000];
        gchar token1[]="\n";
        gchar token2[]=";";
        FILE *inputFilePointer;
        int inputFileSize;
        int i;
	int res1;
	int res2;
	int result;

/* Load the configuration file */
	if (strcmp(setDebug,"TRUE")==0) {
		printf("Loading config.txt...\n");
	}

	strcpy(inputFileName,"config.txt");
	if ((inputFilePointer = fopen(inputFileName,"r")) == NULL) {
		result = showError("\nI can't find the configuration file.");	
		exit;
	}	
	inputFileSize = 0;
        while (fscanf(inputFilePointer,"%c",&C) != -1) text[inputFileSize++] = C;
	fclose(inputFilePointer);

        res1=doSplit(text,token1);
	strcpy(ftpsite,array[0]);
	strcpy(manifestsite,array[1]);
	strcpy(setDebug,array[2]);
	strcpy(cleanUp,array[3]);
	strcpy(internetInstall,array[4]);
	strcpy(iconInstall,array[5]);
	if (strcmp(setDebug,"TRUE")==0) {
		printf("setDebug = %s\n",setDebug);
	}

/* Next get the manifest information into a data structure */
	if (strcmp(setDebug,"TRUE")==0) {
		printf("Getting package manifest\n");
	}
	if (strcmp(internetInstall,"TRUE")==0) {
		strcpy(inputFileName,pkgManifest);
	} else {
		strcpy(inputFileName,localManifest);
	}

	if ((inputFilePointer = fopen(inputFileName,"r")) == NULL) {
		if (strcmp(internetInstall,"FALSE")==0) {
			result = showError("\nI can't find the package manifest on the CD.\nResetting configuration to install from the internet.\nYou may need to mount the CD before continuing.");	
			strcpy(internetInstall,"TRUE");
			strcpy(inputFileName,pkgManifest);
			if ((inputFilePointer = fopen(inputFileName,"r")) == NULL) {
				result = showError("\nI can't find any package manifest!\nMake sure that package_manifest.txt is in the directory\n /opt/ezinstaller-0.1.2/");
				saveprefs();
				exit;
			}
		} else {
			result = showError("\nI can't find the package manifest in the default directory.\nResetting configuration to install from the CD.");	
			strcpy(internetInstall,"FALSE");
			strcpy(inputFileName,localManifest);
			if ((inputFilePointer = fopen(inputFileName,"r")) == NULL) {
				result = showError("\nI can't find any package manifest!\nMake sure that package_manifest.txt is in the directory\n /opt/ezinstaller-0.1.2/");
				exit;
			}
		}				
	}	
        inputFileSize = 0;
        while (fscanf(inputFilePointer,"%c",&C) != -1) text[inputFileSize++] = C;
	fclose(inputFilePointer);


        res1=doSplit(text,token1); 

	if (res1 == 0) {
		if (strcmp(setDebug,"TRUE")==0) {
			printf("No data in file");
		}
	}

	arraySize = res1;
	if (strcmp(setDebug,"TRUE")==0) {
		printf("Lines = %i\n",res1);
	}
	strcpy(manifest_version,array[0]); 
	printf("%s\n",manifest_version);
        for (i=2; i< res1-1; i++) {
		if (strcmp(setDebug,"TRUE")==0) {
			printf("%i: Getting data...",i);
		}
                res2=doSplitLine(array[i],token2);
		if (res2 == 0) {
			if (strcmp(setDebug,"TRUE")==0) {
				printf("No more data");
			}
		}
		/* Consider using version numbers in package_manifest.txt file and branching accordingly */
		if (strcmp(manifest_version,"0.1.2")==0) {
	                strcpy(programs[i].cat,array1[0]);
        	        strcpy(programs[i].meth,array1[1]);
			strcpy(programs[i].loc,array1[2]);
			strcpy(programs[i].md5,array1[3]);
        	        strcpy(programs[i].xpix,array1[4]);
        	        strcpy(programs[i].text,array1[5]);
        	        strcpy(programs[i].where,array1[6]);
		} else {	/* oldest manifest version 0.1.1 */
	                strcpy(programs[i].cat,array1[0]);
        	        strcpy(programs[i].meth,array1[1]);
			strcpy(programs[i].loc,"none");
			strcpy(programs[i].md5,array1[2]);
        	        strcpy(programs[i].xpix,array1[3]);
        	        strcpy(programs[i].text,array1[4]);
        	        strcpy(programs[i].where,array1[5]);
		}
		if (strcmp(setDebug,"TRUE")==0) {
			printf("%s: Line %i OK...\n",programs[i].text,i);
		}
        }

/* Next see what packages are currently installed and put this into a data structure */
	if (strcmp(setDebug,"TRUE")==0) {
		printf("Loading list of installed packages...\n");
	}
	system("./listpackages.sh");
	strcpy(inputFileName,"installed_packages.txt");

	if ((inputFilePointer = fopen(inputFileName,"r")) == NULL) {
		result = showError("\nI can't find the list of installed packages.");	
		exit;
	}	
	inputFileSize = 0;
        while (fscanf(inputFilePointer,"%c",&C) != -1) text[inputFileSize++] = C;
	fclose(inputFilePointer);

        res1=doSplit(text,token1);
	nPackages = res1-1;
	for (i=0; i< nPackages; i++) {
		strcpy(currentPackages[i].where,array[i]);
	}

/* Next load up the packages in the pkglist.txt file */
	if (strcmp(setDebug,"TRUE")==0) {
		printf("Loading general list of packages...\n");
	}
	if (strcmp(internetInstall,"TRUE")==0) {
		strcpy(inputFileName,pkgList);
	} else {
		strcpy(inputFileName,localPkgList);
	}
	if ((inputFilePointer = fopen(inputFileName,"r")) == NULL) {
		if (strcmp(internetInstall,"FALSE")==0) {
			result = showError("\nI can't find the package list.\nResetting configuration to install from the internet.");	
			strcpy(inputFileName,pkgList);
			saveprefs();
			if ((inputFilePointer = fopen(inputFileName,"r")) == NULL) {
				result = showError("\nI still can't find the package list.\nMake certain that the file pkglist.txt is in\n /opt/ezinstaller-0.1.2/");
				exit;
			}
		} else {
			result = showError("\nI can't find the package list.\nResetting configuration to install from CDROM.");	
			strcpy(inputFileName,localPkgList);
			if ((inputFilePointer = fopen(inputFileName,"r")) == NULL) {
				result = showError("\nI still can't find the package list.\nMake certain that the file pkglist.txt is in\n /opt/ezinstaller-0.1.2/");
				exit;
			}
		}		
	}	
	inputFileSize = 0;
        while (fscanf(inputFilePointer,"%c",&C) != -1) text[inputFileSize++] = C;
	fclose(inputFilePointer);

        res1=doSplit(text,token1);
	allPackages = res1-1;
	for (i=0; i< allPackages; i++) {
		strcpy(ftpPackages[i].where,array[i]);
	}

}
 

/* Get and install chosen package after checking to see if any of its parts are
 * already installed.
 */
static void installFTPpackages_4 (GtkWidget *widget,
			gpointer data)
{
/* Add memory here */
	int i;
	gchar *ptext[10000];	/* Text to be written to file current_packages.txt */
	gchar text[10000]= " "; /* Real array that ptext points to */
	gchar *prmtxt[10000]; 	/* Text to be written to file removed_packages.txt */
	gchar rmtxt[10000];	/* Real array the prmtxt points to */
	char fname[100] = "current_packages.txt";
	char rfname[100] = "removed_packages.txt";
	gchar where[200] = " ";	
	gchar *pwhere[200];
	gchar *ppackage[100];	/* List of packages that are already installed */
	gchar package[100] = " "; /* Real array...etc... */
	gchar *ppackname[100];
	gchar packname[100] = " ";
	gchar packname1[100];
	gchar oldpack[100] = " ";
	gchar *poldpack[100];
	gchar temp[500] = " ";
	int res=0;
	int result=0;
	int count=0;
	int rcount=0; 
	int okRemove=0; 
	int pkgCount=0;

	GtkWidget *window2;
	GtkWidget *label;

	/* Initialize pointers */
	*ptext = text;
	*prmtxt = rmtxt;
	*ppackname = packname;
	*ppackage = package;
	*poldpack = oldpack;
	*pwhere = where;

	
	for (i=0;i< ftpsize; i++) {
		if (strcmp(setDebug,"TRUE")==0) {
			printf("ftpout[%i] = %s\n",i,ftpout[i].where);
		}
		res = 0;
		if (strcmp(ftpout[i].where,"null") == 0) {
			/* Do nothing */
		} else {
			/* first determine what the actual package names are */
			strcpy(where,ftpout[i].where);
			if (strcmp(setDebug,"TRUE")==0) {
				printf("where = %s\n",where);
			}

			if (strcmp(internetInstall,"TRUE")==0) {
				*pwhere = strrchr(where, '/');
				strcpy(packname,*pwhere);
				substr(packname,packname1,1);
				if (strcmp(setDebug,"TRUE")==0) {
					printf("packname = %s\n",packname1);
				}
				strcpy(package,packname1); 
				*ppackage =strtok(package,"-"); 
			} else {
				/* if not internetInstall, packname1 and packname are the same */
				strcpy(packname1,where);
				strcpy(packname,where);
			}
				

			/* compare package names to see if the same or similar package */
			isPackageInstalled(packname1,"TRUE");
			okInstall = packageOK.response;
			*poldpack = currentPackages[packageOK.which].where;
			
			/* Then get the packages in question */
			if (strcmp(internetInstall,"TRUE")==0) {
				if (okInstall == 1) {	
					okRemove = 1;
					/* Replace the old file TODO */
					strcpy(temp,"xterm -geometry 70x10 -e wget --passive-ftp --directory-prefix=packages ");
					strcat(temp, ftpout[i].where);
					if (strcmp(setDebug,"TRUE")==0) {
						printf("Replacing package....\n");
					}
					if (count == 0) {
						strcpy(text,packname);
					} else {
						strcat(text,"\n");
						strcat(text,packname);
					}
					count++;
					if (rcount == 0) {
						strcpy(*prmtxt,*poldpack);
					} else {
						strcat(*prmtxt,"\n");
						strcat(*prmtxt,*poldpack);
					}
					rcount++;
					pkgCount++;
					system(temp); 
				} else if (okInstall == 2) { 
					/* Just install the package */
					strcpy(temp,"xterm -geometry 70x10 -e wget --passive-ftp --directory-prefix=packages ");
					strcat(temp, ftpout[i].where);
					if (strcmp(setDebug,"TRUE")==0) {
						printf("Getting package....\n");
					}
					if (count == 0) {
						strcpy(text,packname);
					} else {
						strcat(text,"\n");
						strcat(text,packname);
					}
					count++;
					pkgCount++;
					system(temp); 
				} else {		
					/* Don't get the package or install it */
					if (strcmp(setDebug,"TRUE")==0) {
						printf("Skipping package\n");
					}
				}
			} else {		/* local installation from CD */
				if (okInstall == 1) {	
					okRemove = 1;
					/* Replace the old file TODO */
					if (strcmp(setDebug,"TRUE")==0) {
						printf("Replacing package....\n");
					}
					if (count == 0) {
						strcpy(text,packname);
					} else {
						strcat(text,"\n");
						strcat(text,packname);
					}
					count++;
					if (rcount == 0) {
						strcpy(*prmtxt,*poldpack);
					} else {
						strcat(*prmtxt,"\n");
						strcat(*prmtxt,*poldpack);
					}
					rcount++;
					pkgCount++;
				} else if (okInstall == 2) { 
					/* Just install the package */
					strcat(temp, ftpout[i].where);
					if (strcmp(setDebug,"TRUE")==0) {
						printf("Getting package....\n");
					}
					if (count == 0) {
						strcpy(text,packname);
					} else {
						strcat(text,"\n");
						strcat(text,packname);
					}
					count++;
					pkgCount++;
				} else {		
					/* Don't get the package or install it */
					if (strcmp(setDebug,"TRUE")==0) {
						printf("Skipping package\n");
					}
				}

			}
			if (strcmp(setDebug,"TRUE")==0) {
				printf("%s\n",text);
			}
		}
	}
	if (pkgCount > 0) {
		if (okRemove == 1) {
			writeFile(rfname,*prmtxt);
		} 
		if (strcmp(setDebug,"TRUE")==0) {
			printf("text = %s\n",text);
		}
		writeFile(fname,text);	/* Writes file current_packages.txt */
		if (strcmp(internetInstall,"TRUE")==0) {
			system("./xinstallpackages.sh");
			if (strcmp(cleanUp,"TRUE")==0) {
				system("./cleanup.sh");
			}
		} else {
			system("./xinstall_localpackages.sh");
		}
		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "Installation Finished");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);

		if (strcmp(iconInstall,"TRUE")==0) {
/*			printf("packageLoc vs none = %i\n",strcmp(packageLoc,"none"));
			printf("packageLoc = %s\n",packageLoc);
			printf("packageIcon = %s\n",packageIcon);
			printf("packageName = %s\n",packageName); */
			deblank(packageLoc);
			if (strcmp(packageLoc,"none")!=0) {
				gtk_widget_set_size_request (window2, 350, 120);
	
				label = gtk_label_new ("\nDo you want to add desktop icons?\n\nClicking YES will install icons for KDE and IceWM.\nIf you'd prefer another icon, you can change it later.");
				gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );
				gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
				gtk_widget_show (label);

				gtk_dialog_add_button (GTK_DIALOG (window2), "YES", 1);
				gtk_dialog_add_button (GTK_DIALOG (window2), "NO", 2);
				result = gtk_dialog_run ( GTK_DIALOG (window2) );
				gtk_widget_destroy (window2);

				if (result == 1) {
					deblank(packageIcon);
					makeKDEIcon(packageLoc,packageIcon,packageName);
					makeIceWMIcon(packageLoc,packageName);
				}
			}
		} else {
			gtk_widget_set_size_request (window2, 250, 80);

			label = gtk_label_new ("\nAll finished.  Click OK to continue.");
			gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );
			gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
			gtk_widget_show (label);

			gtk_dialog_add_button (GTK_DIALOG (window2), "OK", 1);
			gtk_dialog_run ( GTK_DIALOG (window2) );
			gtk_widget_destroy (window2);
		}
	}
	doInstall_1(widgetArray[1].widget,widgetArray[1].data);	
}

static void listFTPpackages_3 (GtkWidget *widget,
			gpointer data)
{
	static GtkWidget *window2;
	GtkWidget *scrolled_window;
	GtkWidget *table;
	GtkWidget *button;
	GtkWidget *label;
	gchar *tempName;

	int i;
	int a; 
	int b; 
	int res;
	int boxh;
        gchar token1[]=",";
	gchar descript[500];
/* Add memory here */
	/* where needs to match the size of programs[i].where */
	gchar where[3500];
	gchar progName[100];

	widgetArray[3].widget = widget;
	widgetArray[3].data = data;

	strcpy(where,programs[(int) data].where);
	
	strcpy(progName,programs[(int) data].text);
	tempName = strtok(progName,":");	/* Get the program name from the text string */
	strcpy(packageName,tempName);
	strcpy(packageLoc,programs[(int) data].loc);
	strcpy(packageIcon,programs[(int) data].xpix);

	if (strcmp(setDebug,"TRUE")==0) {
		printf("Separator=%s	Data=%s\n",token1,where); 
	}
	res=doSplitLine(where,token1); 
	for (i=0; i< res; i++) {
		if (strcmp(setDebug,"TRUE")==0) {
			printf("%s\n",array1[i]);
		}
	} 
	if (res == 0) {
		boxh = 50 + 120;
	} else if (res == 1) {
		boxh = 50 + 120;
	} else {
		boxh = 30*res + 130;
	}
	window2= gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (window2), "Download & Install");
	gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
	gtk_widget_set_size_request (window2, 550, boxh);

	label = gtk_label_new ("Below are the packages that may need to be installed.\nUnless you know better, you should install all of them.\nIf you do not want to install a package simply unselect it before clicking INSTALL");
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
	gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), scrolled_window, 
			TRUE, TRUE, 0);
	gtk_widget_show (scrolled_window);
    
    /* create a table of res by 1 squares. */
	if (res == 0) {
		table = gtk_table_new (1, 1, FALSE);
	} else {
		for (i=0; i< res; i++) {
			strcpy(descript,array1[i]);
			deblank(descript);
			strcpy(ftparray[i].where, descript);
			if (strcmp(setDebug,"TRUE")==0) {
				printf("ftparray[%i] = %s\n",i,ftparray[i].where);
			}
		}	
		table = gtk_table_new (res, 1, FALSE);
	}
    
    /* set the spacing to 1 on x and 1 on y */
	gtk_table_set_row_spacings (GTK_TABLE (table), 1);
	gtk_table_set_col_spacings (GTK_TABLE (table), 1);
   
    /* pack the table into the scrolled window */
	gtk_scrolled_window_add_with_viewport (
                   GTK_SCROLLED_WINDOW (scrolled_window), table);
	gtk_widget_show (table);
  
	if (res==0) {
		i=0;
		button = gtk_check_button_new_with_label((char *) data);
		g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (toggle_button_callback_4), (int *) i);

		gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,0,1);
		gtk_widget_show (button);
	} else {
		a=0;
		b=1;
		ftpsize=0;
		for (i=0; i< res; i++) {
			ftpsize++;
/*			printf("Row number %i, %s\n",a,descript); */
			button = gtk_check_button_new_with_label(ftparray[i].where);
			g_signal_connect (G_OBJECT (button), "clicked",
	                      G_CALLBACK (toggle_button_callback_4), (int *) i);
			gtk_toggle_button_set_active((GtkToggleButton *) button, TRUE );
			gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,a,b);
	        	gtk_widget_show (button);
			a++;b++;
		}
	}

/* Add an "install" button to the bottom of the dialog */
	button = gtk_button_new_with_label ("Install");
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
				G_CALLBACK (installFTPpackages_4), (gpointer) "Installing");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window2);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window2)->action_area), button, TRUE, TRUE, 0);
	gtk_widget_show (button);

/* Add a "close" button to the bottom of the dialog */
        button = gtk_button_new_with_label ("Close");
	g_signal_connect (G_OBJECT (button), "clicked",
		G_CALLBACK (listPackageTitles_2), widgetArray[2].data);
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                GTK_SIGNAL_FUNC (gtk_widget_destroy), window2);

/* this makes it so the button is the default. */

        GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window2)->action_area), button, TRUE, TRUE, 0);

/* This grabs this button to be the default button. Simply hitting
     * the "Enter" key will cause this button to activate. */
        gtk_widget_grab_default (button);
        gtk_widget_show (button);

	gtk_widget_show (window2);
	
}


static void listPackageTitles_2 (GtkWidget *widget,
                        gpointer data)
{
	char dataOut[20];
	char descript[200]=" ";
	char thisProg[200]=" ";
	
	gchar xpm[25]=" ";
	gchar temppath[5]=" ";
/*	gchar *result; */
	int i;
	int a;
	int b;
	int nrows=0;
	int res;
	int okInstall;
	
        static GtkWidget *window1;
	GtkWidget *scrolled_window;
	GtkWidget *table;
	GtkWidget *button;
	GtkWidget *box;
	GtkWidget *label;

	widgetArray[2].widget = widget;
	widgetArray[2].data = data;

	strcpy(dataOut,(char *) data);

	window1 = gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (window1), dataOut);
	gtk_container_set_border_width (GTK_CONTAINER (window1), 0);
	gtk_widget_set_size_request (window1, 460, 320);

	/* Create a label for the window */
        label = gtk_label_new ("Select the package\nyou would like to install or upgrade");
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

        /* Pack the image and label into the window */
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), label, FALSE, FALSE, 0);
    	gtk_widget_show (label);

    /* create a new scrolled window. */
	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    
	gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    /* The dialog window is created with a vbox packed into it. */								
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), scrolled_window, 
			TRUE, TRUE, 0);
	gtk_widget_show (scrolled_window);
   
    /* create a table of nrows by 1 squares. */
	for (i=0; i< arraySize; i++) {
		if (strcmp(programs[i].cat,dataOut)==0) {
			nrows++;
		}
	}

	table = gtk_table_new (nrows, 1, FALSE);
   
    /* set the spacing to 1 on x and 1 on y */
	gtk_table_set_row_spacings (GTK_TABLE (table), 1);
	gtk_table_set_col_spacings (GTK_TABLE (table), 1);
    
    /* pack the table into the scrolled window */
	gtk_scrolled_window_add_with_viewport (
                   GTK_SCROLLED_WINDOW (scrolled_window), table);
	gtk_widget_show (table);
   
 	a=0;	/* Row start (a) and finish (b) indices */
	b=1;

	for (i=1; i< arraySize; i++) {
		if (strcmp(programs[i].cat,dataOut)==0) {
			if (newPackage == 1) {
				res=doSplitLine(programs[i].where,",");
				strcpy(thisProg,array1[0]);	/* Get 1st program in list */
				res=doSplitLine(thisProg,"/");
				strcpy(thisProg,array1[res-1]); /* Get last part of path */
			/*	thatProg=strtok(thisProg,"."); */
			/*	printf("Program is = %s\n",thisProg); */
				isPackageInstalled(thisProg,"FALSE");
				okInstall = packageOK.response;
 				if (okInstall == 2) {
					strcpy(temppath,"icons/");
					strcpy(xpm,programs[i].xpix); 
					strcpy(descript,programs[i].text);
					deblank(xpm);
					strcat(temppath,xpm);
			/*		printf("%s\n",temppath);   */
				        button = gtk_button_new ();
		        		g_signal_connect (G_OBJECT (button), "clicked",
		        		                G_CALLBACK (listFTPpackages_3), (gpointer) i);
				        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
		        		                GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
		
		        		box = xpm_label_box ( temppath, descript);
		
		        		gtk_widget_show (box);
		        		gtk_container_add (GTK_CONTAINER (button), box);
					gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,a,b);
		        		gtk_widget_show (button);
		/*			printf("%s\n",programs[i].text); */
					a++; b++;
				}
			} else {
				strcpy(temppath,"icons/");
				strcpy(xpm,programs[i].xpix); 
				strcpy(descript,programs[i].text);
				deblank(xpm);
				strcat(temppath,xpm);
	/*			printf("%s\n",temppath);  */
			        button = gtk_button_new ();
	        		g_signal_connect (G_OBJECT (button), "clicked",
	        		                G_CALLBACK (listFTPpackages_3), (gpointer) i);
			        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
	        		                GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
	
	        		box = xpm_label_box ( temppath, descript);
	
	        		gtk_widget_show (box);
	        		gtk_container_add (GTK_CONTAINER (button), box);
				gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,a,b);
	        		gtk_widget_show (button);
	/*			printf("%s\n",programs[i].text); */
				a++; b++;
			}
		}
	}

	if (a == 0) {	/* This means that no new packages were found */
       		label = gtk_label_new ("No new packages were found.\n\nIf you want to install something new, you might need to\nupdate your package_manifest.txt via the Setup button.");
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );
		gtk_table_attach_defaults (GTK_TABLE(table), label, 0,1,a,b);
		gtk_widget_show(label);
	}
/* Add a "close" button to the bottom of the dialog */
        button = gtk_button_new_with_label ("Close");
	g_signal_connect (G_OBJECT (button), "clicked",
		G_CALLBACK (doInstall_1), (gpointer) widgetArray[1].data);
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);

/* this makes it so the button is the default. */

        GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window1)->action_area), button, TRUE, TRUE, 0);

/* This grabs this button to be the default button. Simply hitting
     * the "Enter" key will cause this button to activate. */
        gtk_widget_grab_default (button);
        gtk_widget_show (button);

	gtk_widget_show (window1);

}

void resetUninstall()
{
	getManifest();

	strcpy(isDialogOpen, (gpointer) "FALSE");
	strcpy(isDialog1OK,"YES");
	strcpy(isDialog2OK,"YES");
	strcpy(isDialog3OK,"YES");
	strcpy(isDialog4OK,"YES");

/*	uninstall_1 (widgetArray[1].widget,widgetArray[1].data); */

}


static void doUninstall (GtkWidget *widget,
			gpointer data)
{

	static GtkWidget *window2;
	GtkWidget *label;
	char labelTxt1[20000] = "\nPackage Description:\n\n";
        gchar inputFileName[50]=" ";
        gchar C;
        gchar text[5000]=" ";
        FILE *inputFilePointer;
        int inputFileSize;
	gchar command1[50] = "./listdescription.sh ";
	gchar command2[50] = "./xuninstallpackage.sh ";

	/* List package description and write it to a file  */
	if (strcmp(setDebug,"TRUE")==0) {
		printf("Current Package = %s\n",currentPackages[(int) data].where);
	}
	strcpy(inputFileName,currentPackages[(int) data].where);
	strcat(command1,inputFileName);
	if (strcmp(setDebug,"TRUE")==0) {
		printf("%s\n",command1);
	}
	strcat(command2,inputFileName);
	system(command1);

/* Open up the package description */
	strcpy(inputFileName,"package_description.txt");
        inputFilePointer = fopen(inputFileName,"r");
        inputFileSize = 0;
        while (fscanf(inputFilePointer,"%c",&C) != -1) text[inputFileSize++] = C;
	fclose(inputFilePointer);

	/* Copy the package description to the label text */
	strcat(labelTxt1,text);

	window2= gtk_dialog_new ();
	gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
	gtk_window_set_title (GTK_WINDOW (window2), "Do You Wish to Uninstall This Package?");
	gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
	gtk_widget_set_size_request (window2, 500, 275);

	label = gtk_label_new (labelTxt1);
/*	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER ); */

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_dialog_add_button (GTK_DIALOG (window2), "Yes", 1);
	gtk_dialog_add_button (GTK_DIALOG (window2), "No", 2);

	gint result = gtk_dialog_run (GTK_DIALOG (window2) );
	if (strcmp(setDebug,"TRUE")==0) {
		printf("Result = %i\n",result);
	}
	if (result == 1) {
		system(command2);
		system("rm package_description.txt");
	}
		
	gtk_widget_destroy (window2);

	/* Confirm package removal */
	window2= gtk_dialog_new ();
	gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
	gtk_window_set_title (GTK_WINDOW (window2), "Alert!");
	gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
	gtk_widget_set_size_request (window2, 250, 80);
	
	label = gtk_label_new ("\n\nPackage Removal Finished\n");
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_dialog_add_button (GTK_DIALOG (window2), "OK", 1);
	gtk_dialog_run ( GTK_DIALOG (window2) );
	gtk_widget_destroy (window2);

	resetUninstall();
/*	getManifest();

	strcpy(isDialogOpen, (char *) data);
	strcpy(isDialog1OK,"YES");
	strcpy(isDialog2OK,"YES");
	strcpy(isDialog3OK,"YES");
	strcpy(isDialog4OK,"YES");

	uninstall_1 (widgetArray[1].widget,widgetArray[1].data);
 */

}


static void uninstall_1 (GtkWidget *widget,
                        gpointer data)
{
	char dataOut[20];
	int i;
	int a;
	int b;
        static GtkWidget *window1;
	GtkWidget *scrolled_window;
	GtkWidget *table;
	GtkWidget *button;
	GtkWidget *box;
	GtkWidget *label;

	if (strcmp(isDialog3OK,"NO")==0) {
		showError("You can only open one first level window at a time.");
		return;
	}
	strcpy(isDialog1OK,"NO");
	strcpy(isDialog2OK,"NO");
	strcpy(isDialog4OK,"NO");

	widgetArray[1].widget = widget;
	widgetArray[1].data = data;

	strcpy(dataOut,(char *) data);

	window1 = gtk_dialog_new ();
	g_signal_connect (G_OBJECT (window1), "destroy",
		      G_CALLBACK (setDialogRules), "uninstall");
	gtk_window_set_title (GTK_WINDOW (window1), dataOut);
	gtk_container_set_border_width (GTK_CONTAINER (window1), 0);
	gtk_widget_set_size_request (window1, 460, 320);

	/* Create a label for the window */
        label = gtk_label_new ("Select the package\nyou would like to uninstall");
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

        /* Pack the image and label into the window */
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), label, FALSE, FALSE, 0);
    	gtk_widget_show (label);

    /* create a new scrolled window. */
	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    
	gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    /* The dialog window is created with a vbox packed into it. */								
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), scrolled_window, 
			TRUE, TRUE, 0);
	gtk_widget_show (scrolled_window);
   
    /* create a table of nPackages by 1 squares. */
	table = gtk_table_new (nPackages, 1, FALSE);
   
    /* set the spacing to 1 on x and 1 on y */
	gtk_table_set_row_spacings (GTK_TABLE (table), 1);
	gtk_table_set_col_spacings (GTK_TABLE (table), 1);
    
    /* pack the table into the scrolled window */
	gtk_scrolled_window_add_with_viewport (
                   GTK_SCROLLED_WINDOW (scrolled_window), table);
	gtk_widget_show (table);
   
 	a=0;
	b=1;

	for (i=0; i< nPackages; i++) {
		button = gtk_button_new ();
		g_signal_connect (G_OBJECT (button), "clicked",
				G_CALLBACK (doUninstall), (gpointer) i);
	        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
		
		box = xpm_label_box("icons/default.xpm",currentPackages[i].where);
		gtk_widget_show (box);
		gtk_container_add (GTK_CONTAINER (button), box);
		gtk_table_attach_defaults (GTK_TABLE(table), button, 0,1,a,b);
		gtk_widget_show (button);
		a++; b++;
	}

/* Add a "close" button to the bottom of the dialog */
        button = gtk_button_new_with_label ("Close");
	g_signal_connect (GTK_OBJECT (button), "clicked",
			G_CALLBACK (setDialogRules), "uninstall");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);

/* this makes it so the button is the default. */

        GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window1)->action_area), button, TRUE, TRUE, 0);

/* This grabs this button to be the default button. Simply hitting
     * the "Enter" key will cause this button to activate. */
        gtk_widget_grab_default (button);
        gtk_widget_show (button);

	gtk_widget_show (window1);

}

static void installSingleFTPpackage_3 (GtkWidget *widget,
			gpointer data)
{

	static GtkWidget *window2;
	GtkWidget *label;
	char labelTxt1[200] = " ";
	char rfname[100] = "removed_packages.txt";
	gchar text[100] = " ";
	gchar *ptext[100];
        gchar inputFileName[50]=" ";
	gchar command1[200] = "echo ";
	gchar command2[50] = " > ftp_package.txt";
	gchar command3[200] = "xterm -geometry 70x10 -e wget --passive-ftp --directory-prefix=packages ";
	gchar *oldpackage[50];
	int okInstall = 2;
	int which = -1;

	widgetArray[3].widget = widget;
	widgetArray[3].data = data;

	*ptext = text;
	strcpy(labelTxt1,ftpPackages[(int) data].where);
	/* List package description and write it to a file  */
	if (strcmp(setDebug,"TRUE")==0) {
		printf("Current Package = %s\n",currentPackages[(int) data].where);
	}
	strcpy(inputFileName,ftpPackages[(int) data].where);
	strcat(command1,inputFileName);
	strcat(command1,command2);
	system(command1);
	if (strcmp(setDebug,"TRUE")==0) {
		printf("%s\n",command1);
	}

	isPackageInstalled(inputFileName,"TRUE"); 	/* Check to see if package is already installed */
	okInstall = packageOK.response;
	which = packageOK.which;
	if (which > -1 ) {
		*oldpackage = currentPackages[which].where;
	}
	
	if (okInstall == 1) {		/* Replace the package to be installed */
		strcat(command3,ftpsite);
		strcat(command3,"/");
		strcat(command3,inputFileName);

		*ptext = *oldpackage;

		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "OK to Install?");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 175);

		label = gtk_label_new (labelTxt1);
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER ); 
	
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);
	
		gtk_dialog_add_button (GTK_DIALOG (window2), "Yes", 1);
		gtk_dialog_add_button (GTK_DIALOG (window2), "No", 2);

		gint result = gtk_dialog_run (GTK_DIALOG (window2) );
		if (strcmp(setDebug,"TRUE")==0) {
			printf("Result = %i\n",result);
		}
		if (result == 1) {	/* If user confirms replace, then get package */
			writeFile(rfname,*ptext);
			system(command3);
			if (strcmp(internetInstall,"TRUE")==0) {
				system("./xinstallpackage.sh");
				if (strcmp(cleanUp,"TRUE")==0) {
					system("./cleanup1.sh");
				}
			} else {
				system("./xinstall_localpackage.sh");
			}
		}

	} else if (okInstall == 2) { 	/* Just install the darn thing */

		strcat(command3,ftpsite);
		strcat(command3,"/");
		strcat(command3,inputFileName);

		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "OK to Install?");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 175);

		label = gtk_label_new (labelTxt1);
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER ); 
	
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);
	
		gtk_dialog_add_button (GTK_DIALOG (window2), "Yes", 1);
		gtk_dialog_add_button (GTK_DIALOG (window2), "No", 2);

		gint result = gtk_dialog_run (GTK_DIALOG (window2) );
		if (strcmp(setDebug,"TRUE")==0) {
			printf("Result = %i\n",result);
		}
		if (result == 1) {	/* If user confirms install, then get package */
			system(command3);
			if (strcmp(internetInstall,"TRUE")==0) {
				system("./xinstallpackage.sh");
				if (strcmp(cleanUp,"TRUE")==0) {
					system("./cleanup1.sh");
				}
			} else {
				system("./xinstall_localpackage.sh");
			}
		}
	} else {	/* Don't install it and abort! Notify user! */
		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "Aborting Install");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 100);

		label = gtk_label_new (labelTxt1);
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER ); 
	
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);
	
		gtk_dialog_add_button (GTK_DIALOG (window2), "OK", 1);

		gtk_dialog_run (GTK_DIALOG (window2) );

	}
		
	gtk_widget_destroy (window2);

	if (okInstall < 3 ) {
		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "Alert!");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 80);
	
		label = gtk_label_new ("\n\nInstallation Finished\n");
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);

		gtk_dialog_add_button (GTK_DIALOG (window2), "OK", 1);
		gtk_dialog_run ( GTK_DIALOG (window2) );
		gtk_widget_destroy (window2);


	}
	getManifest();
	doInstall_1(widgetArray[1].widget,widgetArray[1].data);
}


static void listPackageByName_2 (GtkWidget *widget,
                        gpointer data)
{
	int i;
	int a;
	int b;
	int okInstall;
	gchar thisProg[100];
	
        static GtkWidget *window1;
	GtkWidget *scrolled_window;
	GtkWidget *table;
	GtkWidget *button;
	GtkWidget *box;
	GtkWidget *label;
/*	strcpy(dataOut,(char *) data); */

	widgetArray[2].widget = widget;
	widgetArray[2].data = data;

	window1 = gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (window1), "Install Package");
	gtk_container_set_border_width (GTK_CONTAINER (window1), 0);
	gtk_widget_set_size_request (window1, 460, 320);

	/* Create a label for the window */
        label = gtk_label_new ("Select the package\nyou would like to install.\n\nNo dependencies will be checked.");
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

        /* Pack the image and label into the window */
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), label, FALSE, FALSE, 0);
    	gtk_widget_show (label);

    /* create a new scrolled window. */
	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    
	gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    /* The dialog window is created with a vbox packed into it. */								
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), scrolled_window, 
			TRUE, TRUE, 0);
	gtk_widget_show (scrolled_window);
   
    /* create a table of nPackages by 1 squares. */
	table = gtk_table_new (allPackages, 1, FALSE);
   
    /* set the spacing to 1 on x and 1 on y */
	gtk_table_set_row_spacings (GTK_TABLE (table), 1);
	gtk_table_set_col_spacings (GTK_TABLE (table), 1);
    
    /* pack the table into the scrolled window */
	gtk_scrolled_window_add_with_viewport (
                   GTK_SCROLLED_WINDOW (scrolled_window), table);
	gtk_widget_show (table);
   
 	a=0;	/* Row indices (a=start, b=finish) */
	b=1;

	for (i=0; i< allPackages; i++) {
		if (newPackage == 1) {
			strcpy(thisProg,ftpPackages[i].where);
	/*		printf("Program is = %s\n",thisProg); */
			isPackageInstalled(thisProg,"FALSE");
			okInstall = packageOK.response;
			if (okInstall == 2) {
				button = gtk_button_new ();
				g_signal_connect (G_OBJECT (button), "clicked",
					G_CALLBACK (installSingleFTPpackage_3), (gpointer) i);
		        	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
		        	        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
				box = xpm_label_box("icons/default.xpm",ftpPackages[i].where);
				gtk_widget_show (box);
				gtk_container_add (GTK_CONTAINER (button), box);
				gtk_table_attach_defaults (GTK_TABLE(table), button, 0,1,a,b);
				gtk_widget_show (button);
				a++; b++;
			}
		} else {
			button = gtk_button_new ();
			g_signal_connect (G_OBJECT (button), "clicked",
				G_CALLBACK (installSingleFTPpackage_3), (gpointer) i);
	        	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
	        	        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
			box = xpm_label_box("icons/default.xpm",ftpPackages[i].where);
			gtk_widget_show (box);
			gtk_container_add (GTK_CONTAINER (button), box);
			gtk_table_attach_defaults (GTK_TABLE(table), button, 0,1,a,b);
			gtk_widget_show (button);
			a++; b++;
		}
	}

	if (a == 0) {	/* This means that no new packages were found */
       		label = gtk_label_new ("No new packages were found.\n\nIf you want to install something new, you might need to\nupdate your pkglist.txt via the Setup button.");
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );
		gtk_table_attach_defaults (GTK_TABLE(table), label, 0,1,a,b);
		gtk_widget_show(label);
	}
/* Add a "close" button to the bottom of the dialog */
        button = gtk_button_new_with_label ("Close");
	g_signal_connect (G_OBJECT (button), "clicked",
		G_CALLBACK (doInstall_1), widgetArray[1].data);
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);

/* this makes it so the button is the default. */

        GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window1)->action_area), button, TRUE, TRUE, 0);

/* This grabs this button to be the default button. Simply hitting
     * the "Enter" key will cause this button to activate. */
        gtk_widget_grab_default (button);
        gtk_widget_show (button);

	gtk_widget_show (window1);

}


static void browseselect ( GtkWidget *widget,
			GtkFileSelection *fs )
{
	gchar package[100];
	gchar *ppackage[100];
	gchar packname[50];
	gchar *ppackname[50];
	gchar delim[1] = "/";
	static GtkWidget *window2;
	GtkWidget *label;
/*	char labelTxt1[200] = " "; */
	char rfname[100] = "removed_packages.txt";
	gchar text[100] = " ";
	gchar *ptext[100];
        gchar inputFileName[50]=" ";
	gchar command1[200] = "echo ";
	gchar command2[50] = " > ftp_package.txt";
	gchar *oldpackage[50];
	int okInstall = 2;
	int which = -1;
	int res = 0;

	*ptext = text; 		/* Packages to be uninstalled in txt form */
	*ppackname = packname;	/* Actual name of package */
	strcpy(package, gtk_file_selection_get_filename(GTK_FILE_SELECTION (fs)));
	/* List package description and write it to a file  */
	strcat(command1,package);
	strcat(command1,command2);
	system(command1);
	if (strcmp(setDebug,"TRUE")==0) {
		printf("%s\n",command1);
	}

	*ppackage = package;	/* This is the name + the path */
	res = doSplitLine(*ppackage,delim);
	*ppackname = array1[res-1];
	strcpy(inputFileName,*ppackname);

	isPackageInstalled(inputFileName,"TRUE");
	okInstall = packageOK.response;
	which = packageOK.which;
	if (which > -1 ) {
		*oldpackage = currentPackages[which].where;
	}
	
	if (okInstall == 1) {		/* Replace the package to be installed */
		*ptext = *oldpackage;

		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "OK to Install?");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 175);

		label = gtk_label_new (*ppackname);
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER ); 
	
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);
	
		gtk_dialog_add_button (GTK_DIALOG (window2), "Yes", 1);
		gtk_dialog_add_button (GTK_DIALOG (window2), "No", 2);

		gint result = gtk_dialog_run (GTK_DIALOG (window2) );
		if (strcmp(setDebug,"TRUE")==0) {
			printf("Result = %i\n",result);
		}
		if (result == 1) {
			writeFile(rfname,*ptext);
			system("./xinstallhdpackage.sh");
		}

	} else if (okInstall == 2) { 	/* Just install the darn thing */

		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "OK to Install?");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 175);

		label = gtk_label_new (*ppackname);
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER ); 
	
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);
	
		gtk_dialog_add_button (GTK_DIALOG (window2), "Yes", 1);
		gtk_dialog_add_button (GTK_DIALOG (window2), "No", 2);

		gint result = gtk_dialog_run (GTK_DIALOG (window2) );
		if (strcmp(setDebug,"TRUE")==0) {
			printf("Result = %i\n",result);
		}
		if (result == 1) {
			system("./xinstallhdpackage.sh");
		}
	} else {	/* Don't install it and abort! */
		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "Aborting Install");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 100);

		label = gtk_label_new (*ppackname);
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER ); 
	
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);
	
		gtk_dialog_add_button (GTK_DIALOG (window2), "OK", 1);

		gtk_dialog_run (GTK_DIALOG (window2) );

	}
		
	gtk_widget_destroy (window2);
	if (okInstall < 3 ) {
		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "Alert!");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 120);
	
		label = gtk_label_new ("\nInstallation Finished\n");
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);

		gtk_dialog_add_button (GTK_DIALOG (window2), "OK", 1);
		gtk_dialog_run ( GTK_DIALOG (window2) );
		gtk_widget_destroy (window2);


	}

}

static void browseinstall( GtkWidget *widget,
			gpointer data )
{
    GtkWidget *filew;
    /* Create a new file selection widget */
    filew = gtk_file_selection_new ("File selection");
    
    g_signal_connect (G_OBJECT (filew), "destroy",
	              G_CALLBACK (gtk_widget_destroy), NULL);
    /* Connect the ok_button to file_ok_sel function */
    g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		      "clicked", G_CALLBACK (browseselect), (gpointer) filew);
    g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
	                      "clicked", G_CALLBACK (gtk_widget_destroy),
			      G_OBJECT (filew));
    
    /* Connect the cancel_button to destroy the widget */
    g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button),
	                      "clicked", G_CALLBACK (gtk_widget_destroy),
			      G_OBJECT (filew));
    
    gtk_widget_show (filew);
    if (strcmp(setDebug,"TRUE")==0) {
	printf("This will allow the user to install a package by browsing their hd\n");
    }
}

static void destroy( GtkWidget *widget,
                     gpointer   data )
{
    gtk_main_quit ();
}

static void doInstall_1 (GtkWidget *widget,
			gpointer data)
{
    static GtkWidget *window1;
    GtkWidget *scrolled_window;
    GtkWidget *table;
    GtkWidget *button;
    GtkWidget *box;
    GtkWidget *label;
    GtkTooltips *tooltips;

	/* First see if button 0 or button 1 was pressed */
	if (strcmp((char *) data, "new")==0) {
		if (strcmp(isDialog1OK,"NO")==0) {
			showError("You can only open one first level window at a time.");
			return;
		}
		newPackage=1;	/* In this case listPackageTitles_2 will only show new packages */
		strcpy(isDialog2OK,"NO");
		strcpy(isDialog3OK,"NO");
		strcpy(isDialog4OK,"NO");
	} else {
		if (strcmp(isDialog2OK,"NO")==0) {
			showError("You can only open one first level window at a time.");
			return;
		}
		newPackage=0;	/* In this case all packages will be listed */
		strcpy(isDialog1OK,"NO");
		strcpy(isDialog3OK,"NO");
		strcpy(isDialog4OK,"NO");
	}
	widgetArray[1].widget = widget;
	widgetArray[1].data = data;

	/* Now start building widgets */
	window1 = gtk_dialog_new ();
	g_signal_connect (G_OBJECT (window1), "destroy",
		      G_CALLBACK (setDialogRules), "install");
	gtk_window_set_title (GTK_WINDOW (window1), "Program Groups");
	gtk_container_set_border_width (GTK_CONTAINER (window1), 0);
	gtk_widget_set_size_request (window1, 260, 320);

	/* Create a label for the window */
        label = gtk_label_new ("Select the type of package\nyou would like to install or upgrade");
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

        /* Pack the image and label into the window */
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), label, FALSE, FALSE, 0);
    	gtk_widget_show (label);

    /* create a new scrolled window. */
	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    
	gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
    
    /* the policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
     * GTK_POLICY_AUTOMATIC will automatically decide whether you need
     * scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
     * there.  The first one is the horizontal scrollbar, the second, 
     * the vertical. */
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    /* The dialog window is created with a vbox packed into it. */								
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), scrolled_window, 
			TRUE, TRUE, 0);
	gtk_widget_show (scrolled_window);
    
    /* create a table of 12 by 1 squares. */
	table = gtk_table_new (12, 1, FALSE);
    
    /* set the spacing to 1 on x and 1 on y */
	gtk_table_set_row_spacings (GTK_TABLE (table), 1);
	gtk_table_set_col_spacings (GTK_TABLE (table), 1);
    
    /* pack the table into the scrolled window */
	gtk_scrolled_window_add_with_viewport (
                   GTK_SCROLLED_WINDOW (scrolled_window), table);
	gtk_widget_show (table);
    
/* Add Office button */
        button = gtk_button_new ();
        tooltips = gtk_tooltips_new ();
        gtk_tooltips_set_tip (tooltips, button, "List word processors, spreadsheets and the like.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (listPackageTitles_2), (gpointer) "office");
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/office.xpm", "Office and Productivity");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,0,1);
        gtk_widget_show (button);

/* Add Finance button */
        button = gtk_button_new ();
        tooltips = gtk_tooltips_new ();
        gtk_tooltips_set_tip (tooltips, button, "List budget, checkbook, and accounting software.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (listPackageTitles_2), (gpointer) "finance");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/financial.xpm", "Financial Software");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,1,2);
        gtk_widget_show (button);

/* Add Media button */
        button = gtk_button_new ();
        tooltips = gtk_tooltips_new ();
        gtk_tooltips_set_tip (tooltips, button, "List media players, editors and utilities.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (listPackageTitles_2), (gpointer) "media");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/audio.xpm", "Music and Multimedia");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,2,3);
        gtk_widget_show (button);

/* Add Internet Button */
        button = gtk_button_new ();
        tooltips = gtk_tooltips_new ();
        gtk_tooltips_set_tip (tooltips, button, "List browser, file transfer, messenger, and chat clients.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (listPackageTitles_2), (gpointer) "internet");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/internet.xpm", "Internet and Connectivity");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,3,4);
        gtk_widget_show (button);
	
/* Add Graphics Button */
        button = gtk_button_new ();
        tooltips = gtk_tooltips_new ();
        gtk_tooltips_set_tip (tooltips, button, "List graphics editors, viewers and utilities.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (listPackageTitles_2), (gpointer) "graphics");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/graphics.xpm", "Graphics");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,4,5);
        gtk_widget_show (button);
	
/* Add Games Button */
        button = gtk_button_new ();
        tooltips = gtk_tooltips_new ();
        gtk_tooltips_set_tip (tooltips, button, "List computer games.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (listPackageTitles_2), (gpointer) "games");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/games.xpm", "Games");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,5,6);
        gtk_widget_show (button);

/* Add Science Button */
        button = gtk_button_new ();
        tooltips = gtk_tooltips_new ();
        gtk_tooltips_set_tip (tooltips, button, "List science related programs for learning and professional use.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (listPackageTitles_2), (gpointer) "science");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/science.xpm", "Science");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,6,7);
        gtk_widget_show (button);
	

/* Add Window Manager Button */
        button = gtk_button_new ();
        tooltips = gtk_tooltips_new ();
        gtk_tooltips_set_tip (tooltips, button, "List components of window managers.  Not recommended for beginners as configuration may not be automatic.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (listPackageTitles_2), (gpointer) "windows");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/wm.xpm", "Window Managers");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,7,8);
        gtk_widget_show (button);

/* Add System Administration Button */
        button = gtk_button_new ();
        tooltips = gtk_tooltips_new ();
        gtk_tooltips_set_tip (tooltips, button, "List system administration utilities including printing and scanning.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (listPackageTitles_2), (gpointer) "system");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/systemadmin.xpm", "System Administration");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,8,9);
        gtk_widget_show (button);

/* Add Other Button */
        button = gtk_button_new ();
        tooltips = gtk_tooltips_new ();
        gtk_tooltips_set_tip (tooltips, button, "List all other categories of programs.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (listPackageTitles_2), (gpointer) "other");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/default.xpm", "Other Applications");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,9,10);
        gtk_widget_show (button);
	
/* Add Browse All Button */
        button = gtk_button_new ();
        tooltips = gtk_tooltips_new ();
        gtk_tooltips_set_tip (tooltips, button, "List all packages available for installation from the internet or CDROM. Installations using this option will not automatically install all the needed libraries, thus this is recommended for advanced users only.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (listPackageByName_2), (gpointer) "other");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/pkgselect.xpm", "Package List");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,10,11);
        gtk_widget_show (button);

/* Add Browse Computer */
        button = gtk_button_new ();
        tooltips = gtk_tooltips_new ();
        gtk_tooltips_set_tip (tooltips, button, "Install a package from your computer.  This will install a single package at a time and will not automatically install any additional libraries that are needed. This is recommeded for advanced users only.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (browseinstall), (gpointer) "other");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/harddrive.xpm", "Browse Computer");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,11,12);
        gtk_widget_show (button);


	
/* Add a "close" button to the bottom of the dialog */
	button = gtk_button_new_with_label ("Close");
	g_signal_connect (GTK_OBJECT (button), "clicked",
			G_CALLBACK (setDialogRules), "install");
	g_signal_connect (G_OBJECT (button), "clicked",
			G_CALLBACK (setDialogIsOpen), (gpointer) "FALSE");
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);

/* this makes it so the button is the default. */
    
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window1)->action_area), button, TRUE, TRUE, 0);
    
/* This grabs this button to be the default button. Simply hitting
     * the "Enter" key will cause this button to activate. */
	gtk_widget_grab_default (button);
	gtk_widget_show (button);
    
	gtk_widget_show (window1);

}

static void saveprefs() 
{
	gchar txtOut[1000]=" ";

	strcpy(txtOut,ftpsite);	/* Default location of files to be downloaded from pkglist.txt */
	strcat(txtOut,"\n");
	strcat(txtOut,manifestsite);
	strcat(txtOut,"\n");
	strcat(txtOut,setDebug);	/* Boolean String TRUE= setDebug on, FALSE= off */
	strcat(txtOut,"\n");
	strcat(txtOut,cleanUp);
	strcat(txtOut,"\n");
	strcat(txtOut,internetInstall);
	strcat(txtOut,"\n");
	strcat(txtOut,iconInstall);
/*	printf("config.txt = %s",txtOut); */
	writeFile("config.txt",txtOut);
	
}

static void updateManifest_3 (GtkWidget *widget,
			GtkWidget *data) 
{
	static GtkWidget *window3;
	const gchar *site;
	GtkWidget *label;
	char labelTxt1[1000] = "Default location of manifest server is now:\n";

	site = gtk_entry_get_text (GTK_ENTRY (data));
	strcat(labelTxt1,site);
	strcat(labelTxt1,"\n\n");
	strcat(labelTxt1,"Press OK to accept or Cancel to reject.");

	window3= gtk_dialog_new ();
	gtk_window_set_modal(GTK_WINDOW(window3), TRUE);
	gtk_window_set_title (GTK_WINDOW (window3), "Manifest Server");
	gtk_container_set_border_width (GTK_CONTAINER (window3), 0);
	gtk_widget_set_size_request (window3, 600, 150);

	label = gtk_label_new (labelTxt1);
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window3)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_dialog_add_button (GTK_DIALOG (window3), "OK", 1);
	gtk_dialog_add_button (GTK_DIALOG (window3), "Cancel", 2);

	gint result = gtk_dialog_run (GTK_DIALOG (window3) );
	if (strcmp(setDebug,"TRUE")==0) {
		printf("Result = %i\n",result);
	}

	gtk_widget_destroy (window3);
	if (result == 1) {
	  strcpy(manifestsite,site);
	  if (strcmp(setDebug,"TRUE")==0) {
		  printf ("Entry contents: %s\n", manifestsite);
	  }
	  saveprefs();
	}
}

static void updatePkgList (GtkWidget *widget,
			GtkWidget *data) 
{
	static GtkWidget *window3;
	const gchar *site;
	GtkWidget *label;
	char labelTxt1[1000] = "Default location of manifest server is now:\n";

	site = gtk_entry_get_text (GTK_ENTRY (data));
	strcat(labelTxt1,site);
	strcat(labelTxt1,"\n\n");
	strcat(labelTxt1,"Press OK to accept or Cancel to reject.");

	window3= gtk_dialog_new ();
	gtk_window_set_modal(GTK_WINDOW(window3), TRUE);
	gtk_window_set_title (GTK_WINDOW (window3), "Package List Server");
	gtk_container_set_border_width (GTK_CONTAINER (window3), 0);
	gtk_widget_set_size_request (window3, 600, 150);

	label = gtk_label_new (labelTxt1);
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window3)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_dialog_add_button (GTK_DIALOG (window3), "OK", 1);
	gtk_dialog_add_button (GTK_DIALOG (window3), "Cancel", 2);

	gint result = gtk_dialog_run (GTK_DIALOG (window3) );
	if (strcmp(setDebug,"TRUE")==0) {
		printf("Result = %i\n",result);
	}

	gtk_widget_destroy (window3);
	if (result == 1) {
	  strcpy(ftpsite,site);
	  if (strcmp(setDebug,"TRUE")==0) {
		  printf ("Entry contents: %s\n", manifestsite);
	  }
	  saveprefs();
	}
}


static void setmanifest_2 (GtkWidget *widget,
			gpointer data)
{
    GtkWidget *entry1;
    GtkWidget *entry2;
    GtkWidget *label;
    GtkWidget *window2;
    GtkWidget *box2;
    GtkWidget *button;
    /*GtkWidget *separator; */
 

    widgetArray[2].widget = widget;
    widgetArray[2].widget = widget;
 
    window2 = gtk_dialog_new ();
  
    g_signal_connect (G_OBJECT (window2), "delete_event",
			GTK_SIGNAL_FUNC (gtk_widget_destroy), window2);
    g_signal_connect (GTK_OBJECT (window2), "destroy",
			G_CALLBACK (setup_1), widgetArray[1].data);

    gtk_window_set_title (GTK_WINDOW (window2), "Set Manifest Server");
    gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
    gtk_widget_set_size_request (window2, 500, 220);

/* Create a label for the window */
    label = gtk_label_new ("\nChoose the servers for the package manifest\nand the package list.\nPress RETURN in a text field to update the values.");
    gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

/* Pack the image and label into the window */
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
   		gtk_widget_show (label);

    box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
    gtk_box_pack_start(GTK_BOX (GTK_DIALOG(window2)->vbox),box2, TRUE, TRUE, 0);
    gtk_widget_show (box2);


    label = gtk_label_new ("Internet address of package_manifest.txt");
    gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 0);
    gtk_widget_show (label);

    entry1 = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry1), 100);
    g_signal_connect (G_OBJECT (entry1), "activate",
		      G_CALLBACK (updateManifest_3),
		      (gpointer) entry1);
    gtk_entry_set_text (GTK_ENTRY (entry1), manifestsite);
    gtk_box_pack_start (GTK_BOX (box2), entry1, FALSE, FALSE, 0);
    gtk_widget_show (entry1);

    label = gtk_label_new ("Internet address of pkglist.txt");
    gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 0);
    gtk_widget_show (label);

    entry2= gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry2), 100);
    g_signal_connect (G_OBJECT (entry2), "activate",
		      G_CALLBACK (updatePkgList),
		      (gpointer) entry2);
    gtk_entry_set_text (GTK_ENTRY (entry2), ftpsite);
    gtk_box_pack_start (GTK_BOX (box2), entry2, FALSE, FALSE, 0);
    gtk_widget_show (entry2);


/* Add a "close" button to the bottom of the dialog */
	button = gtk_button_new_with_label ("Close");
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (gtk_widget_destroy), window2);

/* this makes it so the button is the default. */
    
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window2)->action_area), button, TRUE, TRUE, 0);
    
/* This grabs this button to be the default button. Simply hitting
     * the "Enter" key will cause this button to activate. */
	gtk_widget_grab_default (button);
	gtk_widget_show (button);
   
	gtk_widget_show (window2);

}

static void getmanifest_2 (GtkWidget *widget,
			gpointer data)
{
	static GtkWidget *window3;
	GtkWidget *label;
	char labelTxt1[1000] = "\nAre you sure you want to update your manifests?\nThis requires an internet connection.";
	char command1[1000] = " ";

	window3= gtk_dialog_new ();
	gtk_window_set_modal(GTK_WINDOW(window3), TRUE);
	gtk_window_set_title (GTK_WINDOW (window3), "Update Manifests?");
	gtk_container_set_border_width (GTK_CONTAINER (window3), 0);
	gtk_widget_set_size_request (window3, 300, 120);

	label = gtk_label_new (labelTxt1);
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window3)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_dialog_add_button (GTK_DIALOG (window3), "OK", 1);
	gtk_dialog_add_button (GTK_DIALOG (window3), "Cancel", 2);

	gint result = gtk_dialog_run (GTK_DIALOG (window3) );
	if (strcmp(setDebug,"TRUE")==0) {
		printf("Result = %i\n",result);
	}

	gtk_widget_destroy (window3);
	if (result == 1) {
		strcpy(command1,"./xgetmanifests.sh");
		strcat(command1," ");
		strcat(command1,manifestsite);
		strcat(command1," ");
		strcat(command1,ftpsite);
		if (strcmp(setDebug,"TRUE")==0) {
			printf("The command is: %s\n",command1);
		}
		system(command1);
		getManifest();
	}

}

static void localManifestSelect_3 (GtkWidget *widget,
			GtkFileSelection *fs)
{
	gchar fname[100];
	gchar command1[100] = "cp ";
	gchar labelTxt1[100] = "\nThis may change which packages you can install.";
	GtkWidget *label;
	static GtkWidget *window3;

	strcpy(fname,gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
	strcat(command1,fname);
	strcat(command1," ./package_manifest.txt");

	window3= gtk_dialog_new ();
	gtk_window_set_modal(GTK_WINDOW(window3), TRUE);
	gtk_window_set_title (GTK_WINDOW (window3), "Change Manifest?");
	gtk_container_set_border_width (GTK_CONTAINER (window3), 0);
	gtk_widget_set_size_request (window3, 300, 120);

	label = gtk_label_new (labelTxt1);
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window3)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_dialog_add_button (GTK_DIALOG (window3), "OK", 1);
	gtk_dialog_add_button (GTK_DIALOG (window3), "Cancel", 2);

	gint result = gtk_dialog_run (GTK_DIALOG (window3) );
	if (strcmp(setDebug,"TRUE")==0) {
		printf("Result = %i\n",result);
	}

	gtk_widget_destroy (window3);
	if (result == 1) {
		system(command1);
		getManifest();
	}


}

void callSetup () 
{
   	setup_1(widgetArray[1].widget,widgetArray[1].data);
}

static void chooseLocalManifest_2 (GtkWidget *widget,
			gpointer data)
{
    static GtkWidget *filew;
    /* Create a new file selection widget */
    filew = gtk_file_selection_new ("File selection");
    
    g_signal_connect (G_OBJECT (filew), "destroy",
	              G_CALLBACK (gtk_widget_destroy), NULL);
    g_signal_connect (GTK_OBJECT (filew), "destroy",
			G_CALLBACK (callSetup), NULL);
    /* Connect the ok_button to file_ok_sel function */
    g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		      "clicked", G_CALLBACK (localManifestSelect_3), (gpointer) filew);
    g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
	                      "clicked", G_CALLBACK (gtk_widget_destroy),
			      G_OBJECT (filew));
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew), "manifests/");
    
    /* Connect the cancel_button to destroy the widget */
    g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button),
	                      "clicked", G_CALLBACK (gtk_widget_destroy),
			      G_OBJECT (filew));
    
    gtk_widget_show (filew); 
   	/*setup_1(widgetArray[1].widget,widgetArray[1].data); */

}

void installpref_toggle_button_callback_3 (GtkWidget *widget, gpointer data)
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
    {
        /* If control reaches here, the toggle button is down */
	strcpy(internetInstall,"TRUE");

    } else {
        /* If control reaches here, the toggle button is up */
	strcpy(internetInstall,"FALSE");
    }
	/*printf("internetInstall = %s\n",internetInstall); */
}

void iconcreate_toggle_button_callback_3 (GtkWidget *widget, gpointer data)
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
    {
        /* If control reaches here, the toggle button is down */
	strcpy(iconInstall,"TRUE");
    } else {
        /* If control reaches here, the toggle button is up */
	strcpy(iconInstall,"FALSE");
    }
	/*printf("iconInstall = %s\n",iconInstall);*/
}

void debug_toggle_button_callback_3 (GtkWidget *widget, gpointer data)
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
    {
        /* If control reaches here, the toggle button is down */
	strcpy(setDebug,"TRUE");
    } else {
        /* If control reaches here, the toggle button is up */
	strcpy(setDebug,"FALSE");
    }
	/*printf("setDebug = %s\n",setDebug);*/
}

void returnOtherPreferences_2 () 
{
	saveprefs();
	getManifest();
	setup_1(widgetArray[1].widget,widgetArray[1].data);
}

static void setOtherPreferences_2 (GtkWidget *widget,
			gpointer data) 
{
        static GtkWidget *window2;
        GtkWidget *button;
	GtkWidget *separator;

	window2 = gtk_dialog_new ();
    	g_signal_connect (G_OBJECT (window2), "delete_event",
			GTK_SIGNAL_FUNC (gtk_widget_destroy), window2);
	g_signal_connect (GTK_OBJECT (window2), "destroy",
			G_CALLBACK (returnOtherPreferences_2), NULL);
	gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
	gtk_window_set_title (GTK_WINDOW (window2), "Set Other Preferences");
	gtk_container_set_border_width (GTK_CONTAINER (window2), 10);
	gtk_widget_set_size_request (window2, 480, 380);

/* Create a new checkbox */

	button = gtk_check_button_new_with_label("Install from internet. If unchecked, the install will use a \nCDROM for the installation.");
		g_signal_connect (G_OBJECT (button), "clicked",
                G_CALLBACK (installpref_toggle_button_callback_3), NULL);
	if (strcmp(internetInstall,"TRUE")==0) {
		gtk_toggle_button_set_active((GtkToggleButton *) button, TRUE );
	}
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), button, TRUE, TRUE, 0);
        gtk_widget_show (button);

	separator = gtk_hseparator_new ();
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox),separator,TRUE,TRUE,0);
	gtk_widget_show(separator);

/* Create a new checkbox */

	button = gtk_check_button_new_with_label("Install Icons to Desktop when Appropriate. If unchecked, no icons \nwill be installed on your desktop when you install new programs.");
	g_signal_connect (G_OBJECT (button), "clicked",
                G_CALLBACK (iconcreate_toggle_button_callback_3), NULL);
	if (strcmp(iconInstall,"TRUE")==0) {
		gtk_toggle_button_set_active((GtkToggleButton *) button, TRUE );
	}
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), button, TRUE, TRUE, 0);
        gtk_widget_show (button);

	separator = gtk_hseparator_new ();
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox),separator,TRUE,TRUE,0);
	gtk_widget_show(separator);

/* Create a new checkbox */

	button = gtk_check_button_new_with_label("Show Debugging Info when running from the console.\nIf unchecked, no debugging info will be shown.");
	g_signal_connect (G_OBJECT (button), "clicked",
                G_CALLBACK (debug_toggle_button_callback_3), NULL);
	if (strcmp(setDebug,"TRUE")==0) {
		gtk_toggle_button_set_active((GtkToggleButton *) button, TRUE );
	}
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), button, TRUE, TRUE, 0);
        gtk_widget_show (button);


/* Add a "close" button to the bottom of the dialog */
	button = gtk_button_new_with_label ("Close");
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (gtk_widget_destroy), window2);

/* this makes it so the button is the default. */
    
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window2)->action_area), button, TRUE, TRUE, 0);
    
/* This grabs this button to be the default button. Simply hitting
 * the "Enter" key will cause this button to activate. */
	gtk_widget_grab_default (button);
	gtk_widget_show (button);
   
	gtk_widget_show (window2);

}

static void localPkglistSelect_3 (GtkWidget *widget,
			GtkFileSelection *fs)
{
	gchar fname[100];
	gchar command1[100] = "cp ";
	gchar labelTxt1[100] = "\nThis may change which packages you can install.";
	GtkWidget *label;
	static GtkWidget *window3;

	strcpy(fname,gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
	strcat(command1,fname);
	strcat(command1," ./pkglist.txt");

	window3= gtk_dialog_new ();
	gtk_window_set_modal(GTK_WINDOW(window3), TRUE);
	gtk_window_set_title (GTK_WINDOW (window3), "Change Package List?");
	gtk_container_set_border_width (GTK_CONTAINER (window3), 0);
	gtk_widget_set_size_request (window3, 300, 120);

	label = gtk_label_new (labelTxt1);
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window3)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_dialog_add_button (GTK_DIALOG (window3), "OK", 1);
	gtk_dialog_add_button (GTK_DIALOG (window3), "Cancel", 2);

	gint result = gtk_dialog_run (GTK_DIALOG (window3) );
	if (strcmp(setDebug,"TRUE")==0) {
		printf("Result = %i\n",result);
	}

	gtk_widget_destroy (window3);
	if (result == 1) {
		system(command1);
		getManifest();
	}


}

static void chooseLocalPkglist_2 (GtkWidget *widget,
			gpointer data)
{
    GtkWidget *filew;
    /* Create a new file selection widget */
    filew = gtk_file_selection_new ("File selection");
    
    g_signal_connect (G_OBJECT (filew), "destroy",
	              G_CALLBACK (gtk_widget_destroy), NULL);
    g_signal_connect (GTK_OBJECT (filew), "destroy",
			G_CALLBACK (callSetup), NULL);
    /* Connect the ok_button to file_ok_sel function */
    g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		      "clicked", G_CALLBACK (localPkglistSelect_3), (gpointer) filew);
    g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
	                      "clicked", G_CALLBACK (gtk_widget_destroy),
			      G_OBJECT (filew));
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew), "lists/");
    
    /* Connect the cancel_button to destroy the widget */
    g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button),
	                      "clicked", G_CALLBACK (gtk_widget_destroy),
			      G_OBJECT (filew));
    
    gtk_widget_show (filew);


}


static void setup_1 (GtkWidget *widget,
			gpointer data)
{
    static GtkWidget *window1;
    GtkWidget *scrolled_window;
    GtkWidget *table;
    GtkWidget *button;
    GtkWidget *box;
    GtkWidget *label;
    GtkTooltips *tooltips;


	if (strcmp(isDialog4OK,"NO")==0) {
		showError("You can only open one first level window at a time.");
		return;
	}
	strcpy(isDialog1OK,"NO");
	strcpy(isDialog2OK,"NO");
	strcpy(isDialog3OK,"NO");

	widgetArray[1].widget = widget;
	widgetArray[1].data = data;

	/* Now start building widgets */

	window1 = gtk_dialog_new ();
	g_signal_connect (G_OBJECT (window1), "destroy",
		      G_CALLBACK (setDialogRules), "TRUE");
	gtk_window_set_title (GTK_WINDOW (window1), "Configure");
	gtk_container_set_border_width (GTK_CONTAINER (window1), 0);
	gtk_widget_set_size_request (window1, 300, 360);

	/* Create a label for the window */
        label = gtk_label_new ("\nChoose the settings you wish to modify below.\n");
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

        /* Pack the image and label into the window */
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), label, FALSE, FALSE, 0);
    	gtk_widget_show (label);

    /* create a new scrolled window. */
	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    
	gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
    
    /* the policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
     * GTK_POLICY_AUTOMATIC will automatically decide whether you need
     * scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
     * there.  The first one is the horizontal scrollbar, the second, 
     * the vertical. */
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    /* The dialog window is created with a vbox packed into it. */								
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), scrolled_window, 
			TRUE, TRUE, 0);
	gtk_widget_show (scrolled_window);
    
    /* create a table of 5 by 1 squares. */
	table = gtk_table_new (5, 1, FALSE);
    
    /* set the spacing to 1 on x and 1 on y */
	gtk_table_set_row_spacings (GTK_TABLE (table), 1);
	gtk_table_set_col_spacings (GTK_TABLE (table), 1);
    
    /* pack the table into the scrolled window */
	gtk_scrolled_window_add_with_viewport (
                   GTK_SCROLLED_WINDOW (scrolled_window), table);
	gtk_widget_show (table);
    
/* Add Set Manifest button */
        button = gtk_button_new ();
	tooltips = gtk_tooltips_new ();
	gtk_tooltips_set_tip (tooltips, button, "The manifest and package list are files that list compatible programs for installation. This button allows you to change the default location from which updates to these files are downloaded.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (setmanifest_2), (gpointer) "Set Manifest");
	g_signal_connect (G_OBJECT (button), "clicked",
			G_CALLBACK (setDialogIsOpen), (gpointer) "TRUE");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/server.xpm", "Set Package Manifest Server");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,0,1);
        gtk_widget_show (button);

/* Add Get Manifest button */
        button = gtk_button_new ();
	tooltips = gtk_tooltips_new ();
	gtk_tooltips_set_tip (tooltips, button, "Update the package manifest and package list files to get the most current listings of available programs.  Requires an internet connection.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (getmanifest_2), (gpointer) "Get Manifest");
	g_signal_connect (G_OBJECT (button), "clicked",
			G_CALLBACK (setDialogIsOpen), (gpointer) "TRUE");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/internet.xpm", "Get Package Manifest from Server");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,1,2);
        gtk_widget_show (button);

/* Add Choose Local Manifest button */
        button = gtk_button_new ();
	tooltips = gtk_tooltips_new ();
	gtk_tooltips_set_tip (tooltips, button, "Change to another package manifest.  This allows you to select an older version in case you wish to install older packages.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (chooseLocalManifest_2), (gpointer) "Local Manifest");
	g_signal_connect (G_OBJECT (button), "clicked",
			G_CALLBACK (setDialogIsOpen), (gpointer) "TRUE");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/pkgman.xpm", "Choose Local Manifest");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,2,3);
        gtk_widget_show (button);

/* Add Choose Local Package List Button */
        button = gtk_button_new ();
	tooltips = gtk_tooltips_new ();
	gtk_tooltips_set_tip (tooltips, button, "Change to another package list.  This allows you to select an older version in case you wish to install older packages.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (chooseLocalPkglist_2), (gpointer) "Local Package List");
	g_signal_connect (G_OBJECT (button), "clicked",
			G_CALLBACK (setDialogIsOpen), (gpointer) "TRUE");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/pkglist.xpm", "Choose Local Package List");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,3,4);
        gtk_widget_show (button);
	
/* Add Set Installation Prefs Button */
        button = gtk_button_new ();
	tooltips = gtk_tooltips_new ();
	gtk_tooltips_set_tip (tooltips, button, "Choose whether to install from internet or CDROM, show debugging information or install icons when appropriate.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (setOtherPreferences_2), (gpointer) "Other Preferences");
	g_signal_connect (G_OBJECT (button), "clicked",
			G_CALLBACK (setDialogIsOpen), (gpointer) "TRUE");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);
        box = xpm_label_box ("icons/prefs.xpm", "Set Other Preferences");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,4,5);
        gtk_widget_show (button);
	
/* Add a "close" button to the bottom of the dialog */
	button = gtk_button_new_with_label ("Close");
	g_signal_connect (G_OBJECT (button), "clicked",
			G_CALLBACK (setDialogIsOpen), (gpointer) "FALSE");
	g_signal_connect (GTK_OBJECT (button), "clicked",
			G_CALLBACK (setDialogRules), "TRUE");
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);

/* this makes it so the button is the default. */
    
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window1)->action_area), button, TRUE, TRUE, 0);
    
/* This grabs this button to be the default button. Simply hitting
     * the "Enter" key will cause this button to activate. */
	gtk_widget_grab_default (button);
	gtk_widget_show (button);
    
	gtk_widget_show (window1);

}


int main( int   argc,
          char *argv[] )
{
    static GtkWidget *window;
    GtkWidget *scrolled_window;
    GtkWidget *table;
    GtkWidget *button;
    GtkWidget *box;
    GtkWidget *label;
    GtkTooltips *tooltips;
    
    gtk_init (&argc, &argv);
    initArr();
    getManifest();
    
    /* Create a new dialog window for the scrolled window to be
     * packed into.  */
    window = gtk_dialog_new ();
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (destroy), NULL);
    gtk_window_set_title (GTK_WINDOW (window), "VL Easy Installer");
    gtk_container_set_border_width (GTK_CONTAINER (window), 0);
    gtk_widget_set_size_request (window, 275, 300);

/* Create a label for the window */
    label = gtk_label_new ("Vector Linux EZInstaller\nClick on a Button to Continue");
    gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

/* Pack the image and label into the window */
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window)->vbox), label, FALSE, FALSE, 0);
    	gtk_widget_show (label);

/* create a new scrolled window. */
    scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    
    gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
    
    /* the policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
     * GTK_POLICY_AUTOMATIC will automatically decide whether you need
     * scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
     * there.  The first one is the horizontal scrollbar, the second, 
     * the vertical. */
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    /* The dialog window is created with a vbox packed into it. */								
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window)->vbox), scrolled_window, 
			TRUE, TRUE, 0);
    gtk_widget_show (scrolled_window);
    
    /* create a table of 3 by 1 squares. */
    table = gtk_table_new (4, 1, FALSE);
    
    /* set the spacing to 1 on x and 1 on y */
    gtk_table_set_row_spacings (GTK_TABLE (table), 1);
    gtk_table_set_col_spacings (GTK_TABLE (table), 1);
    
    /* pack the table into the scrolled window */
    gtk_scrolled_window_add_with_viewport (
                   GTK_SCROLLED_WINDOW (scrolled_window), table);
    gtk_widget_show (table);
    
/* Add Install New button */
        button = gtk_button_new ();
	tooltips = gtk_tooltips_new ();
	gtk_tooltips_set_tip (tooltips, button, "See a listing of packages not already installed on your system.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (doInstall_1), (gpointer) "new");
        box = xpm_label_box ("icons/newpkg.xpm", "Install a New Package");

        /* Pack and show the widgets */
        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,0,1);
        gtk_widget_show (button);

/* Add Upgrade or Repair button */
        button = gtk_button_new ();
	tooltips = gtk_tooltips_new ();
	gtk_tooltips_set_tip (tooltips, button, "See a listing of all compatible packages whether installed or not.\nThis will allow you to reinstall broken packages or upgrade old ones.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (doInstall_1), (gpointer) "repair");
        box = xpm_label_box ("icons/default.xpm", "Upgrade or Repair a Package");

        /* Pack and show the widgets */
        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,1,2);
        gtk_widget_show (button);

/* Add Uninstall Button */
        button = gtk_button_new ();
	tooltips = gtk_tooltips_new ();
	gtk_tooltips_set_tip (tooltips, button, "See a listing of all the packages that can be uninstalled.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (uninstall_1), (gpointer) "Uninstall a Package");
        box = xpm_label_box ("icons/trash.xpm", "Uninstall a Package");

        /* Pack and show the widgets */
        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,2,3);
        gtk_widget_show (button);
	
/* Add Configuration Button */
        button = gtk_button_new ();
	tooltips = gtk_tooltips_new ();
	gtk_tooltips_set_tip (tooltips, button, "Change the default behavior of this program.",NULL);
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (setup_1), (gpointer) "Configure Easy Installer");
        box = xpm_label_box ("icons/setup.xpm", "Configure Easy Installer");

        /* Pack and show the widgets */
        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,3,4);
        gtk_widget_show (button);

    /* Add a "close" button to the bottom of the dialog */
    button = gtk_button_new_with_label ("Close");
    g_signal_connect_swapped (G_OBJECT (button), "clicked",
			      G_CALLBACK (gtk_widget_destroy),
			      G_OBJECT (window));
    
    /* this makes it so the button is the default. */
    
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->action_area), button, TRUE, TRUE, 0);
    
    /* This grabs this button to be the default button. Simply hitting
     * the "Enter" key will cause this button to activate. */
    gtk_widget_grab_default (button);
    gtk_widget_show (button);
    
    gtk_widget_show (window);
    
    gtk_main();
    
    return 0;
}


